/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    /* 
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */
	
	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");
	
	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });
	
	// Event 1 to n bindings
	/* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
	gc.databind.registry.getBinding('CFG_port.$rawData').addStreamingListener(cmd_sender_listener);
	
    gc.databind.registry.bind('widget.ti_widget_label_lpmstatus.label', 'my_xds.outputControlBlock.lpmMode', function(values) {
    console.log(' >>> ' + values);
    if (templateObj) {
        if (values > 0)
        {
            return 'LPM Active';
        }
        else
        {
            return 'LPM Inactive';
        }

    }  
    });
    
    gc.databind.registry.bind('widget.ti_widget_label_activepresettval.label', 'my_xds.outputControlBlock.activePreset', function(values) {
    console.log(' >>> ' + values);
    if (templateObj) {
        if (values === 0)
        {
            return 'PRESET1';
        }
        else
        {
            return 'PRESET2';
        }

    }  
    });
    
    gc.databind.registry.bind('widget.ti_widget_button_startdemo.label', 'my_xds.inputControlBlock.startDemo', function(values) {
    console.log(' >>> ' + values);
    if (templateObj) {
        if (values  ===  0)
        {
            return 'Start';
        }
        else
        {
            return 'Stop';
        }   
        }  
    },
    function(values) {
    console.log(' >>> ' + values);
    if (templateObj) {
        if (values  ==  'Start')
        {
            return 0;
        }
        else
        {
            return 1;
        }

    }  
    });
    
    
    gc.databind.registry.bind('prop.mydump1', 'my_xds.outputControlBlock.echoDataDump', function(values) {
        console.log(' >>> ' + values);
        if (templateObj && values && values.length > 128) {
            var x = [], y = [];  
            for (var i = 0; i < 128; i++) {
                x.push(i);
                y.push(values[i+1]);
            }
            templateObj.$.ti_widget_plot_echoDataDump.data[0].x=x;
            templateObj.$.ti_widget_plot_echoDataDump.data[0].y=y;
            templateObj.$.ti_widget_plot_echoDataDump.redraw();
        }    
        return values;
    });
    

    
    gc.databind.registry.bind('prop.mydump3', 'my_xds.outputControlBlock.ulmResults', function(values) {
        console.log(' >>> ' + values);
        var distanceValueWidgets = [templateObj.$.ti_widget_label_p1obj1,
        templateObj.$.ti_widget_label_p1obj2,
        templateObj.$.ti_widget_label_p1obj3,
        templateObj.$.ti_widget_label_p1obj4,
        templateObj.$.ti_widget_label_p1obj5,
        templateObj.$.ti_widget_label_p1obj6,
        templateObj.$.ti_widget_label_p1obj7,
        templateObj.$.ti_widget_label_p1obj8]
        if (templateObj) {
             for (var i = 0; i < values.length; i++) {
                 if(((values[i].distance/2*0.000001*343) > 11.2 )||((values[i].distance/2*0.000001*343) < templateObj.$.ti_widget_slider_min_distance.value) )
                 {
                     distanceValueWidgets[i].label = "No object...";
                 }
                 else
                 {
                     distanceValueWidgets[i].label = (values[i].distance/2*0.000001*343).toFixed(4);
                 }
            }           
        }    
        return values;
    });

    templateObj.$.ti_widget_sider_samplingtime._valueChanged = onSamplingTime;
    templateObj.$.ti_widget_slider_min_distance._valueChanged = onMinDistance;
    templateObj.$.ti_widget_checkbox_echodatadump.checkedChanged = onEchoDataDump;
});
/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() 
{
    /* 
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */
	
	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });
	
	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});

/*
*  Boilerplate code for working with components in the application gist
*/


var initComplete = false;
var templateObj;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
	templateObj.addEventListener('dom-change',function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
  	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

        },1);
       setupPlots();
	});
};

var positionPlot = function(plot, display, posIdx) {
    var left = [0, 500, 0, 500, 1000];
    var top = [0, 0, 500, 500, 500];
    // layout.margin = {t:100,b:80,l:80,r:80}
    var width = 480; // initial setting in index.gui file
    var height = 480-160+180;
    plot.layout.autoresize = false;
    plot.layout.height = height;
    plot.layout.width = width;
    if (display) {
        plot.style.display = 'block';
        plot.style.left = left[posIdx]+'px';
        plot.style.top = top[posIdx]+'px';
    } else {
        plot.style.display = 'none';
    }
};

var setupPlots = function() {
    var plotPosIdx=0;
    
    templateObj.$.ti_widget_plot_echoDataDump.data = [
        {type: 'scatter', mode: 'lines'} // data[0] range profile
        ,{type: 'scatter', mode: 'markers'} // data[1] range profile at detected objs
        ,{type: 'scatter', mode: 'lines'} // data[2] noise profile
    ];
    templateObj.$.ti_widget_plot_echoDataDump.layout.title = 'Echo data dump';
    templateObj.$.ti_widget_plot_echoDataDump.layout.xaxis = {
        title: 'Echo Data Dump Samples',
        autorange: false,
        range: [0, 128]
    };
    var ymax = 255;
    templateObj.$.ti_widget_plot_echoDataDump.layout.yaxis = {
        autorange: false,
        range: [0, ymax]
    };
    templateObj.$.ti_widget_plot_echoDataDump.layout.showlegend=false;
    positionPlot(templateObj.$.ti_widget_plot_echoDataDump, true, plotPosIdx++);
    templateObj.$.ti_widget_plot_echoDataDump.redraw();
}

var onRedrawEchoDataDump = function() {
     var update = {x:[],y:[]};
     for (var i = 0; i < 128; i++) {
     update.y.push(i);
     update.x.push(outputControlBlock.echoDataDump[1+i]);
     }
     templateObj.$.ti_widget_plot2.restyle(update, [0,1]);
}

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}

var generateCfg = function() {
    var P = {pga460Cfg: {}, lines: [] };
    P.lines.push('% ***************************************************************');
    
    P.lines.push(['% Communication Mode',templateObj.$.ti_widget_droplist_communication.selectedValue].join(':'));
    P.lines.push(['% Baud Rate',templateObj.$.ti_widget_droplist_baudrate.selectedValue].join(':'));
    P.lines.push(['% AFE Gain',templateObj.$.ti_widget_droplist_AFEgain.selectedValue].join(':'));
    P.lines.push(['% Echo data dump preset',templateObj.$.ti_widget_droplist_echodatadump.selectedValue].join(':'));
    P.lines.push(['% Transducer type',templateObj.$.ti_widget_droplist_transducer.selectedValue].join(':'));
    P.lines.push(['% Number of Objects',templateObj.$.ti_widget_droplist_noofobjects.selectedValue].join(':'));
    P.lines.push(['% Threshold settings preset 1,2',templateObj.$.ti_widget_droplist_preset12_thr.selectedValue].join(':'));
    P.lines.push(['% Time Varying Gain',templateObj.$.ti_widget_droplist_tvgain.selectedValue].join(':'));
    P.lines.push(['% Echo data dump enable',templateObj.$.ti_widget_checkbox_echodatadump.checked ? 1 : 0].join(':'));
    P.lines.push(['% Burn EEPROM',templateObj.$.ti_widget_checkbox_burnEEPROM.checked ? 1 : 0].join(':'));
    P.lines.push(['% System Diagnostics',templateObj.$.ti_widget_checkbox_sysdiag.checked ? 1 : 0].join(':'));
    P.lines.push(['% Sampling time',templateObj.$.ti_widget_sider_samplingtime.value].join(':'));
    P.lines.push(['% Minimum Distance',templateObj.$.ti_widget_slider_min_distance.value].join(':'));
    P.lines.push('% ***************************************************************');
    
    P.pga460Cfg.commMode             = templateObj.$.ti_widget_droplist_communication.selectedValue;
    P.pga460Cfg.baudRate             = templateObj.$.ti_widget_droplist_baudrate.selectedValue;
    P.pga460Cfg.afeGain              = templateObj.$.ti_widget_droplist_AFEgain.selectedValue;
    P.pga460Cfg.tvGain               = templateObj.$.ti_widget_droplist_tvgain.selectedValue;
    P.pga460Cfg.echoDataDumpPreset   = templateObj.$.ti_widget_droplist_echodatadump.selectedValue;
    P.pga460Cfg.transducerType       = templateObj.$.ti_widget_droplist_transducer.selectedValue;
    P.pga460Cfg.preset12Threshold    = templateObj.$.ti_widget_droplist_preset12_thr.selectedValue;
    P.pga460Cfg.enableEchoDataDump   = templateObj.$.ti_widget_checkbox_echodatadump.checked ? 1 : 0;
    P.pga460Cfg.enableBurnEEPROM     = templateObj.$.ti_widget_checkbox_burnEEPROM.checked ? 1 : 0;
    P.pga460Cfg.enableSysDiag        = templateObj.$.ti_widget_checkbox_sysdiag.checked ? 1 : 0;
    P.pga460Cfg.samplingTime         = templateObj.$.ti_widget_sider_samplingtime.value;
    P.pga460Cfg.minDistance          = templateObj.$.ti_widget_slider_min_distance.value;
    P.pga460Cfg.noOfObjects          = templateObj.$.ti_widget_droplist_noofobjects.selectedValue;

    P.lines.push(['pga460Cfg', P.pga460Cfg.commMode, P.pga460Cfg.baudRate , P.pga460Cfg.afeGain , P.pga460Cfg.tvGain, P.pga460Cfg.echoDataDumpPreset,
                P.pga460Cfg.transducerType, P.pga460Cfg.preset12Threshold , P.pga460Cfg.enableEchoDataDump, P.pga460Cfg.enableBurnEEPROM , P.pga460Cfg.enableSysDiag, P.pga460Cfg.samplingTime  ,P.pga460Cfg.minDistance , P.pga460Cfg.noOfObjects].join(' '));
                
    return P;
};

var parseandLoadCfg = function(lines, platform) {
    var P = {pga460Cfg: {}};

    for (var idx=0; idx<lines.length; idx++) {
        var tokens = lines[idx].split(/\s+/);
        if (tokens[0] == 'pga460Cfg') {
            templateObj.$.ti_widget_droplist_communication.selectedValue = parseInt(tokens[1]);
            templateObj.$.ti_widget_droplist_baudrate.selectedValue = parseInt(tokens[2]);
            templateObj.$.ti_widget_droplist_AFEgain.selectedValue = parseInt(tokens[3]);
            templateObj.$.ti_widget_droplist_tvgain.selectedValue = parseInt(tokens[4]);
            templateObj.$.ti_widget_droplist_echodatadump.selectedValue = parseInt(tokens[5]);
            templateObj.$.ti_widget_droplist_transducer.selectedValue = parseInt(tokens[6]);
            templateObj.$.ti_widget_droplist_preset12_thr.selectedValue = parseInt(tokens[7]);
            reflectCheckbox(templateObj.$.ti_widget_checkbox_echodatadump,parseInt(tokens[8]) ? true : false);
            reflectCheckbox(templateObj.$.ti_widget_checkbox_burnEEPROM,parseInt(tokens[9]) ? true : false);
            reflectCheckbox(templateObj.$.ti_widget_checkbox_sysdiag,parseInt(tokens[10]) ? true : false);
            templateObj.$.ti_widget_sider_samplingtime.value = parseFloat(tokens[11]);
            templateObj.$.ti_widget_slider_min_distance.value = parseFloat(tokens[12]);
            templateObj.$.ti_widget_droplist_noofobjects.selectedValue = parseInt(tokens[13]);           
        }
    }
   
    return P;
};

var onLoadCfg = function() {
    gc.File.browseAndLoad(null, null, function(data,fileInfo,err) {
        var lines = data.replace(/\r\n/g, '\n').split('\n');
        parseandLoadCfg(lines);
    }, myFileLoadDialog);
};

var onSamplingTime = function() {
    templateObj.$.ti_widget_samplingtime.label = templateObj.$.ti_widget_sider_samplingtime.value;
};

var onEchoDataDump = function() {
    if(templateObj.$.ti_widget_checkbox_echodatadump.checked  === true)
    {
        positionPlot(templateObj.$.ti_widget_plot_echoDataDump, true);
    }
    else
    {
        positionPlot(templateObj.$.ti_widget_plot_echoDataDump, false);
    }
};

var onMinDistance = function() {
    templateObj.$.ti_widget_mindis.label = templateObj.$.ti_widget_slider_min_distance.value;
};

var onStart = function() {
    if(templateObj.$.ti_widget_button_startdemo.label == 'Start')
    {
        templateObj.$.ti_widget_button_startdemo.label = 'Stop';
    }
    else
    {
        templateObj.$.ti_widget_button_startdemo.label = 'Start';
    }
};

var reflectCheckbox = function(widget, newValue) {
    //if (newValue && widget.checked != newValue) {
        widget.checked = newValue;
        return true;
    //}
    //return false;
};

var onResetProfile = function() {
    //templateObj.$.ti_widget_input_range_res_sel.value = Input.Range_Resolution;
    templateObj.$.ti_widget_droplist_communication.selectedIndex = 0;
    templateObj.$.ti_widget_droplist_baudrate.selectedIndex = 4;
    templateObj.$.ti_widget_droplist_AFEgain.selectedIndex = 3;
    templateObj.$.ti_widget_droplist_echodatadump.selectedIndex = 1;
    templateObj.$.ti_widget_droplist_transducer.selectedIndex = 0;
    templateObj.$.ti_widget_droplist_noofobjects.selectedIndex = 3;
    templateObj.$.ti_widget_droplist_preset12_thr.selectedIndex = 1;
    templateObj.$.ti_widget_slider_min_distance.value  = .3;
    templateObj.$.ti_widget_sider_samplingtime.value = 200;
    reflectCheckbox(templateObj.$.ti_widget_checkbox_sysdiag,true);
    reflectCheckbox(templateObj.$.ti_widget_checkbox_echodatadump,true);
    reflectCheckbox(templateObj.$.ti_widget_checkbox_burnEEPROM,false);
}

var onSaveCfg = function() {
    var cfg = generateCfg();
    var delim = '\n';
    var tmp = window.navigator.platform;
    if (tmp) {
        tmp = tmp.toLowerCase();
        if (tmp.indexOf('win') >= 0) delim = '\r\n';
    }
    var data = cfg.lines.join(delim);
    gc.File.saveBrowserFile(data, {filename: 'profile.cfg'}, function(e1) {
        // don't have any callback
    });
};

var cmd_sender_listener = {

    clearConsole: function() {
        templateObj.$.ti_widget_textbox_cfg_console.value = '';
    },
    appendConsole: function(msg) {
        if (templateObj.$.ti_widget_textbox_cfg_console.value.length > 10000)
            this.clearConsole();
        if (templateObj.$.ti_widget_textbox_cfg_console.value.length > 0)
            templateObj.$.ti_widget_textbox_cfg_console.value += '\n' + msg;
        else
            templateObj.$.ti_widget_textbox_cfg_console.value = msg;
        templateObj.$.ti_widget_textbox_cfg_console.scrollTop = 999999;
    },
    onDataReceived: function(data) {
        if (!data) return;
        // expect \r\n  or \n as delimters.  \n\r is strange. The gc backplane delimits by \n, which is good.
        this.appendConsole(data);
    },
};
